SFX_VOLUME=$0f
BGM_VOLUME=$0b

.segment "PESKYTONE_ZP" : zeropage

PV_NOTE_TABLE:		.res 2		;word
PV_WAVE_PTN:		.res 1		;byte
PV_WAVE_PREV:		.res 2		;2 bytes



VIC20_LOW=$900a			;base=cpuclock/16, hz=base/(127-val), bit 0..6 divider, bit 7 output enable
VIC20_MID=$900b			;base=cpuclock/8
VIC20_HIGH=$900c		;base=cpuclock/4
VIC20_NOISE=$900d		;base=cpuclock/32
VIC20_VOLUME=$900e		;bits 0..3 global volume 0 to 15 (silence to max), bits 4..7 aux color



	.macro PEST_INIT
	
	jsr pest_detect_rate
	
	ldx #<note_table_ntsc
	ldy #>note_table_ntsc
	ora #0
	beq :+
	ldx #<note_table_pal
	ldy #>note_table_pal
:
	stx <PV_NOTE_TABLE+0
	sty <PV_NOTE_TABLE+1
	
	jsr mute_all
	
	.endmacro
	
	
;------------------------------------------------------------------------------
	
	
	.macro PEST_DONE
	
	jsr mute_all
	
	lda VIC20_VOLUME
	and #$f0
	sta VIC20_VOLUME
	
	.endmacro
	
	
;------------------------------------------------------------------------------
	
	
	.macro PEST_DETECT_RATE

	lda #0
	ldx $ede4		;NTSC=$05, PAL=$0C
	cpx #$05
	beq :+
	txa
:
					;0=60Hz, !0=50Hz

	.endmacro
	
	
;------------------------------------------------------------------------------


	.macro PEST_MIX
	
	ldx #(CHANNELS_ALL-1)*CH_STRUCT_SIZE
	
@loop:

	lda PV_CHANNELS+CH_WAVE,x
	bne @output
	
	txa
	sec
	sbc #CH_STRUCT_SIZE*2
	tax
	bne @loop

@output:

	
	ldy PV_CHANNELS+CH_WAVE,x
	lda PV_CHANNELS+CH_NOTE_OUT,x
	ldx #0
	jsr set_pitch_and_wave

	lda PV_SFX_PENDING
	and #$7f
	cmp #$7f
	bne :+
	
	ldx #1*CH_STRUCT_SIZE
	ldy PV_CHANNELS+CH_WAVE,x
	lda PV_CHANNELS+CH_NOTE_OUT,x
	sec
	sbc #12
	ldx #1
	jsr set_pitch_and_wave
	
	lda #BGM_VOLUME
	jmp :++
:
	lda #$7e
	sta VIC20_MID
	sta PV_WAVE_PREV+1
	lda #SFX_VOLUME
:

	sta PV_TEMP
	lda VIC20_VOLUME
	and #$f0
	ora PV_TEMP
	sta VIC20_VOLUME
	
	.endmacro
	
	
;------------------------------------------------------------------------------

	.macro PEST_NOTE_TABLE
	
	
mute_all:

	lda #0
	sta VIC20_LOW
	sta VIC20_MID
	sta VIC20_HIGH
	sta VIC20_NOISE
	lda VIC20_VOLUME
	rts
	

	
set_pitch_and_wave:

	pha	
	tya
	bne :+
	
	sta <PV_WAVE_PREV,x
	lda #$7e
	sta VIC20_LOW,x
	pla
	rts
:
	
	asl a
	tay
		
	lda waveform_table,y
	iny
	
	.ifdef PT_VIZNUT_WAVEFORMS
	
	cmp PV_WAVE_PREV,x
	beq @skip_wave
	
	sta PV_WAVE_PREV,x
	sta PV_WAVE_PTN
	
	lda #$7e
	sta VIC20_LOW,x
	
	lda waveform_table,y
	
	ldy #100/5				;delay t-states/5
:
	dey
	bne :-

	tay

@loop:

	lda <PV_WAVE_PTN		;3
	and #$80				;2
	ora wavecnt_table,x		;4+
	sta VIC20_LOW,x			;4+
	asl <PV_WAVE_PTN		;5
	lda <PV_WAVE_PTN		;3
	nop						;2
	nop						;2
	nop						;2
	dey						;2
	bne @loop				;2/3=32t

@skip_wave:

	.endif
	
	pla
	tay
	lda (PV_NOTE_TABLE),y
	sta VIC20_LOW,x
	
	rts
	
	
wavecnt_table:

	.byte $7d,$7b,$79
	
waveform_table:

	.byte %11000000,3	;00
	.byte %10010000,6	;14
	.byte %10100000,5	;09
	.byte %10100000,4	;12
	.byte %10110000,6	;11
	.byte %10110000,5	;07
	.byte %10000000,2	;01
	.byte %10000000,3	;04
	.byte %11010000,5	;08
	.byte %10101000,6	;13
	.byte %10010000,5	;06
	.byte %11100000,4	;03
	.byte %10110000,4	;02
	.byte %11000000,5	;10
	.byte %10000000,4	;05
	.byte %00000000,0	;15
	 
note_table_pal:
	.byte $b0,$b4,$b8,$bc,$c0,$c4,$c7,$ca
	.byte $cd,$d0,$d2,$d5,$d7,$d9,$db,$dd
	.byte $df,$e1,$e3,$e4,$e6,$e7,$e8,$ea
	.byte $eb,$ec,$ed,$ee,$ef,$f0,$f1,$f1
	.byte $f2,$f3,$f3,$f4,$f5,$f5,$f6,$f6
	.byte $f7,$f7,$f8,$f8,$f8,$f9,$f9,$f9
	.byte $fa,$fa,$fa,$fa,$fb,$fb,$fb,$fb
note_table_ntsc:
	.byte $b6,$ba,$be,$c1,$c5,$c8,$cb,$ce
	.byte $d1,$d3,$d6,$d8,$da,$dc,$de,$e0
	.byte $e2,$e3,$e5,$e6,$e8,$e9,$ea,$eb
	.byte $ec,$ed,$ee,$ef,$f0,$f1,$f2,$f2
	.byte $f3,$f4,$f4,$f5,$f5,$f6,$f6,$f7
	.byte $f7,$f8,$f8,$f8,$f9,$f9,$f9,$fa
	.byte $fa,$fa,$fa,$fb,$fb,$fb,$fb,$fb


	.endmacro
