;register aliases

PPU_CTRL	=$2000
PPU_MASK	=$2001
PPU_STATUS	=$2002
PPU_OAM_ADDR=$2003
PPU_OAM_DATA=$2004
PPU_SCROLL	=$2005
PPU_ADDR	=$2006
PPU_DATA	=$2007
PPU_OAM_DMA	=$4014
DMC_FREQ	=$4010
CTRL_PORT1	=$4016
CTRL_PORT2	=$4017

OAM_BUF		=$0200
PAL_BUF		=$01c0

;gamepad readback bits aliases

PAD_A  		=$01
PAD_B		=$02
PAD_SELECT	=$04
PAD_START	=$08
PAD_UP		=$10
PAD_DOWN	=$20
PAD_LEFT	=$40
PAD_RIGHT	=$80



.segment "ZEROPAGE"

NTSC_MODE: 		.res 1
FRAME_CNT1: 	.res 1
FRAME_CNT2: 	.res 1

PAD_STATE:		.res 1
PAD_STATEP:		.res 1
PAD_STATET:		.res 1

PTR_ZP:			.res 2
OFF_ZP:			.res 2

MUS_PAUSED:		.res 2



.segment "HEADER"

NES_MAPPER=0
NES_PRG_BANKS=1
NES_CHR_BANKS=1
NES_MIRRORING=0

    .byte $4e,$45,$53,$1a
	.byte <NES_PRG_BANKS
	.byte <NES_CHR_BANKS
	.byte <NES_MIRRORING|(<NES_MAPPER<<4)
	.byte <NES_MAPPER&$f0
	.res 8,0



.segment "STARTUP"

start:
_exit:

    sei
    ldx #$ff
    txs
    inx
    stx PPU_MASK
    stx DMC_FREQ
    stx PPU_CTRL		;no NMI

initPPU:

    bit PPU_STATUS
@1:
    bit PPU_STATUS
    bpl @1
@2:
    bit PPU_STATUS
    bpl @2

clearPalette:

	lda #$3f
	sta PPU_ADDR
	stx PPU_ADDR
	lda #$0f
	ldx #$20
@1:
	sta PPU_DATA
	dex
	bne @1

clearVRAM:

	txa
	ldy #$20
	sty PPU_ADDR
	sta PPU_ADDR
	ldy #$10
@1:
	sta PPU_DATA
	inx
	bne @1
	dey
	bne @1

clearRAM:

    txa
@1:
    sta $000,x
    sta $100,x
    sta $200,x
    sta $300,x
    sta $400,x
    sta $500,x
    sta $600,x
    sta $700,x
    inx
    bne @1

	lda #%10000000
	sta PPU_CTRL		;enable NMI
	lda #%00000110
	sta PPU_MASK

	jsr wait_frame

detectNTSC:
	ldx #52				;blargg's code
	ldy #24
@1:
	dex
	bne @1
	dey
	bne @1

	lda PPU_STATUS
	and #$80
	sta <NTSC_MODE
	
	
;set palette

	lda #>$3f03
	sta PPU_ADDR
	lda #<$3f03
	sta PPU_ADDR
	lda #$30
	sta PPU_DATA
	
	
;print message
	
	lda #<message
	sta PTR_ZP+0
	lda #>message
	sta PTR_ZP+1

	lda #<$2041
	sta OFF_ZP+0
	lda #>$2000
	sta OFF_ZP+1
	
	ldy #0
	
print1:

	lda OFF_ZP+1
	sta PPU_ADDR
	lda OFF_ZP+0
	sta PPU_ADDR
	
print2:

	lda (PTR_ZP),y
	iny
	bne :+
	inc PTR_ZP+1
:

	cmp #0
	beq print3
	cmp #CR
	bne :+
	
	lda OFF_ZP+0
	clc
	adc #32
	sta OFF_ZP
	bcc print1
	inc OFF_ZP+1
	jmp print1
	
:
	sta PPU_DATA
	jmp print2
	
print3:


;enable rendering, init sound system

	jsr wait_frame

	lda #0
	sta PPU_SCROLL
	sta PPU_SCROLL		
	sta PPU_ADDR
	sta PPU_ADDR
	sta PPU_OAM_ADDR
	
	lda #%00001110
	sta PPU_MASK
	
	jsr pest_init

	
	
loop_main:

	jsr wait_frame
	
	jsr pest_update
	
	jsr pad_poll
	
	lda PAD_STATET
	and #PAD_LEFT
	beq :+
	lda #0
	jsr pest_sound_play
:

	lda PAD_STATET
	and #PAD_RIGHT
	beq :+
	lda #1
	jsr pest_sound_play
:

	lda PAD_STATET
	and #PAD_UP
	beq :+
	lda #2
	jsr pest_sound_play
:

	lda PAD_STATET
	and #PAD_DOWN
	beq :+
	lda #3
	jsr pest_sound_play
:

	lda PAD_STATET
	and #PAD_A
	beq :+
	lda #<music_1_location
	ldx #>music_1_location
	jsr pest_music_load
	jsr pest_music_play
	lda #0
	sta MUS_PAUSED
:

	lda PAD_STATET
	and #PAD_B
	beq :+
	lda #<music_2_location
	ldx #>music_2_location
	jsr pest_music_load
	jsr pest_music_play
	lda #0
	sta MUS_PAUSED
:

	lda PAD_STATET
	and #PAD_SELECT
	beq :+
	jsr pest_music_stop
:

	lda PAD_STATET
	and #PAD_START
	beq :+
	lda MUS_PAUSED
	eor #1
	sta MUS_PAUSED
	jsr pest_music_pause
:

	jmp loop_main



wait_frame:

	lda <FRAME_CNT1
:
	cmp <FRAME_CNT1
	beq :-
	rts
	
	
	
pad_poll:

	ldy #$01
	sty CTRL_PORT1
	dey
	sty CTRL_PORT1
	ldy #8
	
@poll1:

	lda CTRL_PORT1
	and #$01
	clc
	beq @poll2
	sec
	
@poll2:

	ror PAD_STATE
	dey
	bne @poll1

	lda PAD_STATE
	eor PAD_STATEP
	and PAD_STATE
	sta PAD_STATET
	lda PAD_STATE
	sta PAD_STATEP

	rts


	
	
	
	
nmi:
	inc <FRAME_CNT1
irq:
	rti
	
	
	
CR=$0d

message:

	.byte "******************************", CR
	.byte "*                            *", CR
	.byte "*   PeskyTone system test    *", CR
	.byte "*                            *", CR
	.byte "******************************", CR, CR
	.byte " A to start music 1", CR
	.byte " B to start music 2", CR
	.byte " SELECT to stop music", CR
	.byte " START to pause/unpause music", CR
	.byte " U,D,L,R to play effects", CR
	.byte 0
	
music_1_location:
	.incbin "music_1.bin"
music_2_location:
	.incbin "music_2.bin"
	
	.include "peskytone_struct.s"
	.include "peskytone_nes.s"
	.include "peskytone.s"
	.include "peskytone_sfx.s"

	
	
.segment "SAMPLES"
.segment "VECTORS"

    .word nmi	;$fffa vblank nmi
    .word start	;$fffc reset
   	.word irq	;$fffe irq / brk

.segment "CHARS"

	.incbin "nesfont.chr"
